// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file ElementRegionED.h */

#ifndef AMDIS_ELEMENTREGIONED_H
#define AMDIS_ELEMENTREGIONED_H

#include "ElementData.h"
#include "FixVec.h"

namespace AMDiS {

  class ElementRegion_ED : public ElementData
  {
  public:
    inline bool isOfType(int typeID) const 
    {
      if (typeID == ELEMENT_REGION) 
	return true;
      return false;
    }

    class Creator : public CreatorInterface<ElementData>
    {
    public:
      ElementData* create() 
      {
	return new ElementRegion_ED;
      }
    };

    ElementRegion_ED(ElementData *decorated = NULL)
      : ElementData(decorated),
	region(-1)
    {}

    bool refineElementData(Element* parent, 
			   Element* child1,
			   Element* child2,
			   int elType);

    ElementData *clone() const;

    inline std::string getTypeName() const 
    { 
      return "ElementRegion_ED"; 
    }

    inline const int getTypeID() const 
    { 
      return ELEMENT_REGION; 
    }

    void serialize(std::ostream& out);

    void deserialize(std::istream& in); 

    inline void setRegion(int r) 
    { 
      region = r; 
    }

    inline int getRegion() const 
    { 
      return region; 
    }

  protected:
    int region;
  };

}

#endif
