// ============================================================================
// ==                                                                        ==
// == Amdis - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file Spreadsheet.h */

#ifndef AMDIS_SPREADSHEET_H
#define AMDIS_SPREADSHEET_H

#include <vector>
#include <string>

namespace AMDiS {

  using namespace std;

  /** 
   * \brief 
   * Implements basic support to read and write tables of data (all are 
   * assumed to be of type double. The files are written in ASCII mode and
   * allow to use comments (lines that start with #).
   */
  class Spreadsheet
  {
  public:
    typedef vector<vector<double> > DataType;

    Spreadsheet()
      : data(0)
    {}

    void addData(vector<double> &d)
    {
      data.push_back(d);
    }

    void addData(DataType &d)
    {
      data.insert(data.end(), d.begin(), d.end());
    }

    void setData(DataType &d)
    {
      data = d;
    }

    DataType& getData()
    {
      return data;
    }

    void write(string filename);

    void read(string filename);

  protected:
    DataType data;
  };
}

#endif
