// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file PetscSolverCahnHilliard.h */

#ifndef AMDIS_PETSC_SOLVER_CAHN_HILLIARD_H
#define AMDIS_PETSC_SOLVER_CAHN_HILLIARD_H

#include "parallel/PetscSolverGlobalBlockMatrix.h"

namespace AMDiS {

  using namespace std;

  struct CahnHilliardData {
    KSP kspMass, kspLaplace;
    Mat matM, matMinusDeltaK;
    double *eps, *delta; 
    MPI::Intracomm *mpiCommGlobal;

    
  };

  class PetscSolverCahnHilliard : public PetscSolverGlobalBlockMatrix
  {
  public:
    /// Creator class
    class Creator : public OEMSolverCreator
    {
    public:
      virtual ~Creator() {}

      /// Returns a new PetscSolver object.
      OEMSolver* create() 
      { 
	return new PetscSolverCahnHilliard(this->name); 
      }
    };
    
    void solvePetscMatrix(SystemVector &vec, AdaptInfo *adaptInfo);
    
    PetscSolverCahnHilliard(string name, double *epsPtr = NULL, double *deltaPtr = NULL);

    void setChData(double *epsPtr, double *deltaPtr, SystemVector* vec)
    {
      eps = epsPtr;
      delta = deltaPtr;
      solution = vec;
    }

  protected:
    void initSolver(KSP &ksp);

    void initPreconditioner(PC pc);
    void exitPreconditioner(PC pc);
    
    PetscSolver* createSubSolver(int component, string kspPrefix);

  private:
    int pressureComponent;

    PetscSolver *massMatrixSolver, *laplaceMatrixSolver, *deltaKMatrixSolver;

    CahnHilliardData matShellContext;

    bool useOldInitialGuess;
    
    SystemVector* solution;

    double *eps, *delta;
  };

}

#endif // AMDIS_PETSC_SOLVER_CAHN_HILLIARD_H
