/** \file NewtonCotesQuad.h */

#ifndef NEWTON_COTES_QUAD_H
#define NEWTON_COTES_QUAD_H

#include <boost/array.hpp>
#include "Tools.h"

namespace tools {
  
  template<int deg>
  struct NewtonCotesQuad
  {
    typedef boost::array<double,1> Container;
    typedef GeneralQuad<Container> type;

    static type& provide()
    {
      ERROR("ERROR: no, or unknown degree, for quadrature rule given!\n");
      static Container xi = {{0.0}};
      static Container ci = {{1.0}};
      return *(new type(xi,ci));
    }
  };

  template<>
  struct NewtonCotesQuad<1>
  {
    typedef boost::array<double,1> Container;
    typedef GeneralQuad<Container> type;

    static type& provide()
    {
      static Container xi = {{0.5}};
      static Container ci = {{1.0}};
      return *(new type(xi,ci));
    }
  };

  template<>
  struct NewtonCotesQuad<2>
  {
    typedef boost::array<double,2> Container;
    typedef GeneralQuad<Container> type;

    static type& provide()
    {
      static Container xi = {{-0.5/sqrt(3.0)+0.5, 0.5/sqrt(3.0)+0.5}};
      static Container ci = {{0.5, 0.5}};
      return *(new type(xi,ci));
    }
  };

  template<>
  struct NewtonCotesQuad<3>
  {
    typedef boost::array<double,2> Container;
    typedef GeneralQuad<Container> type;

    static type& provide()
    {
      static Container xi = {{-0.5/sqrt(3.0)+0.5, 0.5/sqrt(3.0)+0.5}};
      static Container ci = {{0.5, 0.5}};
      return *(new type(xi,ci));
    }
  };

  template<>
  struct NewtonCotesQuad<4>
  {
    typedef boost::array<double,3> Container;
    typedef GeneralQuad<Container> type;

    static type& provide()
    {
      static Container xi = {{-sqrt(3.0/5.0)/2.0+0.5, 0.5, sqrt(3.0/5.0)/2.0+0.5}};
      static Container ci = {{5.0/18.0, 8.0/18.0, 5.0/18.0}};
      return *(new type(xi,ci));
    }
  };

  template<>
  struct NewtonCotesQuad<5>
  {
    typedef boost::array<double,3> Container;
    typedef GeneralQuad<Container> type;

    static type& provide()
    {
      static Container xi = {{-sqrt(3.0/5.0)/2.0+0.5, 0.5, sqrt(3.0/5.0)/2.0+0.5}};
      static Container ci = {{5.0/18.0, 8.0/18.0, 5.0/18.0}};
      return *(new type(xi,ci));
    }
  };

  template<>
  struct NewtonCotesQuad<6>
  {
    typedef boost::array<double,4> Container;
    typedef GeneralQuad<Container> type;

    static type& provide()
    {
      static Container xi = {{-sqrt(3.0/7.0 + 2.0/7.0 * sqrt(6.0/5.0))/2.0 + 0.5,
			      -sqrt(3.0/7.0 - 2.0/7.0 * sqrt(6.0/5.0))/2.0 + 0.5,
			      sqrt(3.0/7.0 - 2.0/7.0 * sqrt(6.0/5.0))/2.0 + 0.5,
			      sqrt(3.0/7.0 + 2.0/7.0 * sqrt(6.0/5.0))/2.0 + 0.5}};
      static Container ci = {{(18.0-sqrt(30.0))/72.0,
			      (18.0+sqrt(30.0))/72.0,
			      (18.0+sqrt(30.0))/72.0,
			      (18.0-sqrt(30.0))/72.0}};
      return *(new type(xi,ci));
    }
  };

  template<>
  struct NewtonCotesQuad<7>
  {
    typedef boost::array<double,4> Container;
    typedef GeneralQuad<Container> type;

    static type& provide()
    {
      static Container xi = {{-sqrt(3.0/7.0 + 2.0/7.0 * sqrt(6.0/5.0))/2.0 + 0.5,
			      -sqrt(3.0/7.0 - 2.0/7.0 * sqrt(6.0/5.0))/2.0 + 0.5,
			      sqrt(3.0/7.0 - 2.0/7.0 * sqrt(6.0/5.0))/2.0 + 0.5,
			      sqrt(3.0/7.0 + 2.0/7.0 * sqrt(6.0/5.0))/2.0 + 0.5}};
      static Container ci = {{(18.0-sqrt(30.0))/72.0,
			      (18.0+sqrt(30.0))/72.0,
			      (18.0+sqrt(30.0))/72.0,
			      (18.0-sqrt(30.0))/72.0}};
      return *(new type(xi,ci));
    }
  };

} // end namespace

#endif // NEWTON_COTES_QUAD_H
