/** \file BaseProblem_RB.h */

#ifndef BASE_PROBLEM_RB_H
#define BASE_PROBLEM_RB_H

#include "AMDiS.h"
#include "BaseProblem.h"
#include "time/ExtendedRosenbrockStationary.h"

using namespace AMDiS;

/** BaseProblems for Rosenbrock time-discretization
 */

class BaseProblem_RB : public BaseProblem<ExtendedRosenbrockStationary>
{
public: // typedefs

  typedef BaseProblem<ExtendedRosenbrockStationary> super;

// Rosenbrock methods
//_________________________________________________________________________

public:
  
  BaseProblem_RB(std::string name_) : super(name_), minus1(-1.0) {}
  
  void acceptTimestep() { prob->acceptTimestep(); }
  void reset() { prob->reset(); }
  
  void setRosenbrockMethod(RosenbrockMethod *method) { prob->setRosenbrockMethod(method); }
  void setTau(double *ptr) { prob->setTau(ptr); }
  void setTauGamma(double *ptr0, double *ptr1, double *ptr2, double *ptr3, double *ptr4) { prob->setTauGamma(ptr0,ptr1,ptr2,ptr3,ptr4); }

  double* getTauGamma() { return prob->getTauGamma(); }
  double* getMinusTauGamma() { return prob->getMinusTauGamma(); }
  double* getInvTauGamma() { return prob->getInvTauGamma(); }
  double* getMinusInvTauGamma() { return prob->getMinusInvTauGamma(); }
  
protected:
  double minus1;
};

#endif // BASE_PROBLEM_RB_H